class OwllinkController < ApplicationController
	before_filter :check_connected, :only => [:disconnect, :init, :release, :populate, :describe, :set, :settings, :prefixes]
	before_filter :check_initialized, :only => [:release, :populate, :set, :settings, :prefixes]
	before_filter :check_not_initialized, :only => :init
	before_filter :check_not_connected, :only => :connect
	before_filter :release_kb, :only => [:disconnect, :release]
	
    rescue_from ActionController::InvalidAuthenticityToken, :with => :bad_token
    
	def index
        end
	
	def connect
	  begin
		Owllink.connect session
		@request = ""
		@reply = ""
		self.describe
		if @reply && !@reply.blank?
          reply_doc = REXML::Document.new(@reply)   
		
		  reasoner_version = reply_doc.root.elements['/ResponseMessage/Description/ReasonerVersion']
		  Owllink.set_name(session, "#{reply_doc.root.elements[1,'Description'].attributes['name']} #{reasoner_version.attributes['major']}.#{reasoner_version.attributes['minor']}.#{reasoner_version.attributes['build']}")
		  
		  owllink_version = reply_doc.root.elements['/ResponseMessage/Description/ProtocolVersion']
		  Owllink.set_owllink_version(session, "#{owllink_version.attributes['major']}.#{owllink_version.attributes['minor']}")
		  		
		  # check if abbreviatesIRIs is a Setting (an not a Property)
          abbreviatesIRIs_setting = !((REXML::XPath.match reply_doc, "//Setting[@key='abbreviatesIRIs']").blank?)
          Owllink.set_abbreviates_setting(session,abbreviatesIRIs_setting)
        
          # check abbreviatesIRIs status (if not connected as this gives only the default!)
          if !Owllink.initialized?(session)
            if abbreviatesIRIs_setting
              abbreviatesIRIs = (REXML::XPath.first reply_doc, "//Setting[@key='abbreviatesIRIs']/Literal/text()").value() == "true"
            else
              abbreviatesIRIs = (REXML::XPath.first reply_doc, "//Property[@key='abbreviatesIRIs']/Literal/text()").value() == "true"
            end
            Owllink.set_abbreviates_iris(session,abbreviatesIRIs)
          end
          
		  # check if <SupportedExtension identifier="http://www.owllink.org/ext/retraction"/> is part of the Description
		  retraction = REXML::XPath.match reply_doc, "//SupportedExtension[@identifier='http://www.owllink.org/ext/retraction']"
		  Owllink.set_supports_retraction(session,!retraction.blank?)
        end
      rescue Exception => exc
          logger.error("Exception: #{exc.message}")
          flash[:notice] = "Exception: #{exc.message}"
          redirect_to :action => 'index'
      end
    end
	
	def disconnect
	  begin
		Owllink.disconnect(session)
		Owllink.set_name(session,nil)
		Owllink.set_owllink_version(session,nil)
		unless @request && @reply
			flash[:notice] = "Disconnected."
			render :action => 'index'
        end
      rescue Exception => exc
        logger.error("Exception: #{exc.message}")
        flash[:notice] = "Exception: #{exc.message}"
        redirect_to :action => 'index'
      end
    end
	
	def init  
	  begin
		@request1 = Owllink.request_message(:namespaces => {
											:owl => "http://www.w3.org/2002/07/owl#"
											}) {|xml| 
			xml.CreateKB(:name => "#{Owllink.get_server_id session}") {
			  xml.Prefix(:name => "agent", :fullIRI => "http://www.iyouit.eu/agent.owl#")
			  xml.Prefix(:name => "time", :fullIRI => "http://www.iyouit.eu/time.owl#")
			  xml.Prefix(:name => "space", :fullIRI => "http://www.iyouit.eu/space.owl#")
			  xml.Prefix(:name => "situation", :fullIRI => "http://www.iyouit.eu/situation.owl#")
			}
		}
		@reply1 = Owllink.post_request(@request1)
		axioms = File.read("#{RAILS_ROOT}/lib/situation-owllink.xml").to_s + "\n"
		
		reply1_doc = REXML::Document.new(@reply1) 
		kb = reply1_doc.root.elements[1,'KB'].attributes['kb']
		
		if Owllink.load_ontologies?(session)
		  @request2 =  Owllink.request_message(:namespaces => {
											   :owl => "http://www.w3.org/2002/07/owl#"
											   }) {|xml| 
			xml.LoadOntologies(:kb => kb) {
			  xml.OntologyIRI(:IRI => "http://www.iyouit.eu/situation.owl")
			  xml.IRIMapping(:key => "http://www.iyouit.eu/",
			                 :value => "http://cw.docomolab-euro.com/Ontologies/1.8/")
			}
		}
		else
		  @request2 = Owllink.request_message(:namespaces => {
		  									  :owl => "http://www.w3.org/2002/07/owl#"
											  }) {|xml| 
			  xml.Tell(:kb => kb) {
				  xml.text! "#axioms#"
			  }
		  }
		  @request2 = @request2.sub("#axioms#",axioms)		
		end
		
		@reply2 = Owllink.post_request(@request2)
		reply2_doc = REXML::Document.new(@reply2) 
		first_result = reply2_doc.root.elements['/ResponseMessage/*[1]']
		if first_result.name == "OK"
			Owllink.set_kb(session,kb)
			Owllink.set_initialized(session,true)
        end
      rescue Exception => exc
        logger.error("Exception: #{exc.message}")
        flash[:notice] = "Exception: #{exc.message}"
        redirect_to :action => 'index'
      end
    end
	
	def release
		unless @request && @reply
			render :action => 'index'
			end
		end
	
	def populate
	  begin
		kb = Owllink.get_kb session
		@request = Owllink.request_message(:namespaces => {
										   :owl => "http://www.w3.org/2002/07/owl#"
										   }) {|xml|
			xml.GetSubClasses(:kb => kb, :direct => "false") {
				xml.owl(:Class, :abbreviatedIRI => :"space:Place")
			}
			xml.GetSubClasses(:kb => kb, :direct => "false") {
				xml.owl(:Class, :abbreviatedIRI => :"time:Interval_thing")      
			}
			xml.GetSubObjectProperties(:kb => kb, :direct => "false") {
			    xml.owl(:ObjectProperty, :abbreviatedIRI => :"agent:contact")
			}
		}
		@reply = Owllink.post_request(@request)
		reply_doc = REXML::Document.new(@reply)    
		reply_doc.elements.each("/ResponseMessage/*[1]/*/owl:Class") { |element| 
		    abbrev_iri = element.attributes["abbreviatedIRI"]
		    if !abbrev_iri.blank?
			  place = abbrev_iri.split(':')[1]
            else
              place = element.attributes["IRI"].split('#')[1]
            end
			if !PlaceConcept.find_by_iri("space:" + place) and not (place == "Nothing" or place == "Thing")
				new_place = PlaceConcept.new
				new_place.iri = "space:" + place
				new_place.name = place.sub("_"," ").titleize
				new_place.save
            end
		}   
		Rails.logger.info("log 2") 
		reply_doc.elements.each("/ResponseMessage/*[2]/*/owl:Class") { |element|  
		    abbrev_iri = element.attributes["abbreviatedIRI"]
		    if !abbrev_iri.blank?
			  daytime = abbrev_iri.split(':')[1]
            else
              daytime = element.attributes["IRI"].split('#')[1]
            end
			if !DaytimeConcept.find_by_iri("time:" + daytime) and not (daytime == "Nothing" or daytime == "Thing")
				new_daytime = DaytimeConcept.new
				new_daytime.iri = "time:" + daytime
				new_daytime.name = daytime.sub("_"," ").titleize
				new_daytime.save
             end
		} 
		Rails.logger.info("log 3") 
		reply_doc.elements.each("/ResponseMessage/*[3]/*/owl:ObjectProperty") { |element| 
		    abbrev_iri = element.attributes["abbreviatedIRI"]
		    if !abbrev_iri.blank?
			  relation = abbrev_iri.split(':')[1]
            else
              relation = element.attributes["IRI"].split('#')[1]
            end
			if !Relation.find_by_iri("agent:" + relation) and not (relation == "topObjectProperty" or relation == "bottomObjectProperty")
				new_relation = Relation.new
				new_relation.iri = "agent:" + relation
				new_relation.name = relation.sub("_"," ")
				new_relation.save
             end
		}
      rescue Exception => exc
        logger.error("Exception: #{exc.message}")
        flash[:notice] = "Exception: #{exc.message}"
        redirect_to :action => 'index'
      end
    end
    
	def reset  
	  begin
		require 'active_record/fixtures'
		Dir.glob(File.join(RAILS_ROOT, 'test', 'fixtures', '*.{yml,csv}')).each do |fixture_file|
			Fixtures.create_fixtures("test/fixtures",File.basename(fixture_file, '.*'))
			end
		flash[:notice] = "DB initialized."
		redirect_to :action => 'index'
      rescue Exception => exc
        logger.error("Exception: #{exc.message}")
        flash[:notice] = "Exception: #{exc.message}"
        redirect_to :action => 'index'
      end
    end
	
	def describe
	  begin
		@request = Owllink.request_message {|xml| xml.GetDescription}
		@reply = Owllink.post_request(@request)
      rescue Exception => exc
        logger.error("Exception: #{exc.message}")
        flash[:notice] = "Exception: #{exc.message}"
        redirect_to :action => 'index'
      end
    end
	
	def prefixes
	  begin
		@request = Owllink.request_message {|xml| xml.GetPrefixes(:kb => Owllink.get_kb(session)) }
		@reply = Owllink.post_request(@request)
      rescue Exception => exc
        logger.error("Exception: #{exc.message}")
        flash[:notice] = "Exception: #{exc.message}"
        redirect_to :action => 'index'
      end
    end
	
	def set
	 begin
	    if Owllink.abbreviates_setting?(session)
	       abbreviatesIRIs = Owllink.abbreviates_iris?(session)
		   @request = Owllink.request_message {|xml| 
		     xml.Set(:kb => Owllink.get_kb(session), :key => "abbreviatesIRIs") {
		       xml.Literal(abbreviatesIRIs ? "false" : "true")
             }
           }
		   @reply = Owllink.post_request(@request)
           Owllink.set_abbreviates_iris(session,!abbreviatesIRIs)
	    end          
      rescue Exception => exc
        logger.error("Exception: #{exc.message}")
        flash[:notice] = "Exception: #{exc.message}"
        redirect_to :action => 'index'
      end
	end
	
	def settings
	 begin
		@request = Owllink.request_message {|xml| xml.GetSettings(:kb => Owllink.get_kb(session))}
		@reply = Owllink.post_request(@request)
      rescue Exception => exc
        logger.error("Exception: #{exc.message}")
        flash[:notice] = "Exception: #{exc.message}"
        redirect_to :action => 'index'
      end
    end
	

	protected

	def release_kb
	  begin
		@request = nil
		@reply = nil
		if Owllink.initialized? session
			kb = Owllink.get_kb session
			if kb && !kb.empty?
				@request = Owllink.request_message {|xml| 
					xml.ReleaseKB(:kb => kb)
				}
				@reply = Owllink.post_request(@request)
				Owllink.set_kb(session,nil)
                end
			Owllink.set_initialized(session,false)
			end
      rescue Exception => exc
        logger.error("Exception: #{exc.message}")
        flash[:notice] = "Exception: #{exc.message}"
        redirect_to :action => 'index'
      end
    end

	
	def check_connected
	  begin
		unless Owllink.connected?(session)
			flash[:notice] = "Connect first!"
			redirect_to :action => 'index'
			end  
      rescue Exception => exc
        logger.error("Exception: #{exc.message}")
        flash[:notice] = "Exception: #{exc.message}"
        redirect_to :action => 'index'
      end
    end
	
	def check_initialized
	  begin
		unless Owllink.initialized?(session)
			flash[:notice] = "Initialize first!"
			redirect_to :action => 'index'
        end  
      rescue Exception => exc
        logger.error("Exception: #{exc.message}")
        flash[:notice] = "Exception: #{exc.message}"
        redirect_to :action => 'index'
      end
    end
	
	def check_not_initialized
	  begin
		if Owllink.initialized?(session)
			flash[:notice] = "Already initialized."
			redirect_to :action => 'index'
         end  
      rescue Exception => exc
        logger.error("Exception: #{exc.message}")
        flash[:notice] = "Exception: #{exc.message}"
        redirect_to :action => 'index'
      end
    end
	
	def check_not_connected
	  begin
		if Owllink.connected?(session)
			flash[:notice] = "Already connected."
			redirect_to :action => 'index'
        end  
      rescue Exception => exc
        logger.error("Exception: #{exc.message}")
        flash[:notice] = "Exception: #{exc.message}"
        redirect_to :action => 'index'
      end
    end
    
    def bad_token
        flash[:notice] = "Session has expired."
        redirect_to :action => 'index'
    end
end
