class SituationsController < ApplicationController
	before_filter :check_connected, :only => [:type]
	before_filter :check_initialized, :only => [:type]
	
	
	# GET /situations
	# GET /situations.xml
	def index
		@situations = Situation.all
		
		respond_to do |format|
			format.html # index.html.erb
			format.xml  { render :xml => @situations }
			end
		end
	
	# GET /situations/1
	# GET /situations/1.xml
	def show
		@situation = Situation.find(params[:id])
		
		respond_to do |format|
			format.html # show.html.erb
			#format.xml  { render :xml => @situation }
			format.xml  { render :layout => "layouts/owllink_request.xml.erb" }
			end
		end
	
	# GET /situations/new
	# GET /situations/new.xml
	def new
		@situation = Situation.new
		
		respond_to do |format|
			format.html # new.html.erb
			format.xml  { render :xml => @situation }
			end
		end
	
	# GET /situations/1/edit
	def edit
		@situation = Situation.find(params[:id])
		end
	
	# POST /situations
	# POST /situations.xml
	def create
		@situation = Situation.new(params[:situation])
		
		respond_to do |format|
			if @situation.save
				flash[:notice] = 'Situation was successfully created.'
				format.html { redirect_to(@situation) }
				format.xml  { render :xml => @situation, :status => :created, :location => @situation }
				else
				format.html { render :action => "new" }
				format.xml  { render :xml => @situation.errors, :status => :unprocessable_entity }
				end
			end
		end
	
	# PUT /situations/1
	# PUT /situations/1.xml
	def update
		params[:situation][:contact_ids] ||= [] 
		@situation = Situation.find(params[:id])
		
		respond_to do |format|
			if @situation.update_attributes(params[:situation])
				flash[:notice] = 'Situation was successfully updated.'
				format.html { redirect_to(@situation) }
				format.xml  { head :ok }
				else
				format.html { render :action => "edit" }
				format.xml  { render :xml => @situation.errors, :status => :unprocessable_entity }
				end
			end
		end
	
	# DELETE /situations/1
	# DELETE /situations/1.xml
	def destroy
		@situation = Situation.find(params[:id])
		@situation.destroy
		
		respond_to do |format|
			format.html { redirect_to(situations_url) }
			format.xml  { head :ok }
			end
		end
	
	def type
		@situation = Situation.find(params[:id])
		kb = Owllink.get_kb session
		@request = Owllink.request_message(:namespaces => 
		  {:owl => "http://www.w3.org/2002/07/owl#", :ret => "http://www.owllink.org/ext/retraction#"}) {|xml| 
			xml.Tell(:kb => kb) {
				@situation.to_OWL(xml)
			}
			#    <GetTypes kb="situation" direct="true">
			#        <owl:Individual abbreviatedIRI="<situation_iri>"/>
			#    </GetTypes>
			xml.GetTypes(:kb => kb, :direct => "true") {
				xml.owl(:NamedIndividual, :abbreviatedIRI => @situation.iri)
			}
			if Owllink.supports_retraction? session
				xml.ret(:Retract, :kb => kb) {
					@situation.to_OWL(xml)
				}
            else
				xml.ReleaseKB(:kb => kb)
            end
		}
		@reply = Owllink.post_request(@request)    
		reply_doc = REXML::Document.new(@reply)     
		@situation.types = reply_doc.elements.collect("/ResponseMessage/*[2]/*/owl:Class") { |element|
		  if Owllink.abbreviates_iris?(session)
			"situation:" + element.attributes["abbreviatedIRI"].split(':')[1]
		  else
			"situation:" + element.attributes["IRI"].split('#')[1]
          end
		} 
		@situation.save   
		unless Owllink.supports_retraction? session
			Owllink.set_kb(session,nil)
			Owllink.set_initialized(session,false)
        end 
    end
	
	
	
	protected
	
    
	def check_connected
		unless Owllink.connected?(session)
			flash[:notice] = "Connect first!"
			redirect_to :action => 'index'
			end  
		end
	
	def check_initialized
		unless Owllink.initialized?(session)
			flash[:notice] = "Initialize first!"
			redirect_to :action => 'index'
			end  
		end
	
	end
