
class Situation < ActiveRecord::Base
  serialize :types
  belongs_to :place
  belongs_to :daytime
  has_and_belongs_to_many :contacts
  validates_presence_of :name
  validates_uniqueness_of :name
  validates_presence_of :place_id
  validates_presence_of :daytime_id
  
  def iri
    :"situation:s#{self.id}"
  end
  
  def to_OWL(xml)
    # define some identifiers for individuals
    me_iri = :"agent:me"
    # define some identifiers for roles
    company_iri = :"situation:company"    
    location_iri = :"space:location"
    time_iri = :"time:time"
    
    xml.comment! "Define situation #{self.name} individual"
    Owllink.add_class_assertion(xml,"situation:Situation",self.iri)
    xml.comment! "Define me"        
    Owllink.add_class_assertion(xml,"agent:Myself",me_iri)
    for contact in self.contacts
      xml.comment! "Define nearby person #{contact.name}"
      Owllink.add_class_assertion(xml,"agent:Anybody_else",contact.iri)
      xml.comment! "Define relation #{contact.relation.iri} between me and #{contact.name}"
      Owllink.add_objectproperty_assertion(xml,contact.relation.iri,me_iri,contact.iri)
      xml.comment! "Fill #{self.name} company role with #{contact.name}"
      Owllink.add_objectproperty_assertion(xml,company_iri,self.iri,contact.iri)
    end
    if self.contacts && self.contacts.length > 0
      xml.comment! "Close company role"
      Owllink.close_role(xml,self.iri,company_iri,self.contacts.length)
    end
    xml.comment! "Define place individual #{self.place.name}"    
    Owllink.add_class_assertion(xml,place.place_concept.iri,place.iri)
    xml.comment! "Fill #{self.name} location role with #{self.place.name}"
    Owllink.add_objectproperty_assertion(xml,location_iri,self.iri,place.iri)
    xml.comment! "Define daytime individual #{self.daytime.name}"
    Owllink.add_class_assertion(xml,daytime.daytime_concept.iri,daytime.iri)
    xml.comment! "Fill #{self.name} time role with #{self.daytime.name}"
    Owllink.add_objectproperty_assertion(xml,time_iri,self.iri,daytime.iri)
  end
  
  def contact_list
    self.contacts.map {|c| c.name}.uniq.join(',')
  end
  
  def type_list
    (self.types && !self.types.empty? ? self.types.map {|t| t.split(':')[1].sub("_"," ").titleize}.uniq.join(',') : '-')
  end
  
end
